﻿using Hotline.Permissions;
using Hotline.Settings;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Role;
using Identity.Admin.HttpClient;
using Identity.Shared.Dtos.Identity;
using Identity.Shared.Dtos.Role;
using MapsterMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Exceptions;
using XF.Utility.UnifyResponse;

namespace Hotline.Api.Controllers;

/// <summary>
/// 角色权限管理相关接口
/// </summary>
public class RoleController : BaseController
{
    private readonly IIdentityClient _identityClient;
    private readonly ISystemAuthorityRepository _systemAuthorityRepository;
    private readonly ISystemDataAuthorityRepository _systemDataAuthorityRepository;
    private readonly IMapper _mapper;

    public RoleController(IIdentityClient identityClient, 
            ISystemAuthorityRepository systemAuthorityRepository,
            ISystemDataAuthorityRepository systemDataAuthorityRepository,
            IMapper mapper)
    {
        _identityClient = identityClient;
        _systemAuthorityRepository = systemAuthorityRepository;
        _systemDataAuthorityRepository = systemDataAuthorityRepository;
        _mapper = mapper;
    }

    /// <summary>
    /// 分页查询角色
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("paged")]
    [Permission(EPermission.QueryPagedRole)]
    public async Task<PagedDto<IdentityRoleDto>> QueryPaged([FromQuery] QueryRolesPagedDto dto)
    {
        var getRolesRsp = await _identityClient.GetRolesPagedAsync(dto, HttpContext.RequestAborted);
        CheckHttpRequestSuccess(getRolesRsp, "GetRolesPagedAsync");
        var result = getRolesRsp.Result;
        return new PagedDto<IdentityRoleDto>(result.TotalCount, result.Roles);
    }

    /// <summary>
    /// 新增角色
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [Permission(EPermission.AddRole)]
    [HttpPost]
    public async Task<string> Add([FromBody] IdentityRoleDto dto)
    {
        var existsRsp = await _identityClient.IsRoleExistsAsync(dto.Name, HttpContext.RequestAborted);
        CheckHttpRequestSuccess(existsRsp, "IsRoleExistsAsync");
        if (existsRsp.Result)
            throw UserFriendlyException.SameMessage("角色名重复");

        var addRoleRsp = await _identityClient.AddRoleAsync(dto, HttpContext.RequestAborted);
        CheckHttpRequestSuccess(addRoleRsp, "AddRoleAsync");
        return addRoleRsp.Result;
    }

    /// <summary>
    /// 删除角色
    /// </summary>
    /// <param name="roleId"></param>
    /// <returns></returns>
    [Permission(EPermission.RemoveRole)]
    [HttpDelete("{roleId}")]
    public async Task Remove(string roleId)
    {
        var delRoleRsp = await _identityClient.DeleteRoleAsync(roleId, HttpContext.RequestAborted);
        CheckHttpRequestSuccess(delRoleRsp, "DeleteRoleAsync");
        await _systemAuthorityRepository.RemoveAsync(roleId);
        await _systemDataAuthorityRepository.RemoveAsync(x=>x.RoleId== roleId);
    }

    /// <summary>
    /// 更新角色
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [Permission(EPermission.UpdateRole)]
    [HttpPut]
    public async Task Update([FromBody] UpdateRoleDto dto)
    {
        var updateRoleRsp = await _identityClient.UpdateRoleAsync(dto, HttpContext.RequestAborted);
        CheckHttpRequestSuccess(updateRoleRsp, "UpdateRoleAsync");
    }

    #region 应用权限管理

    /// <summary>
    /// 分配权限
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [Permission(EPermission.AllocationAuthority)]
    [HttpPost("allocationauthority")]
    public async Task AllocationAuthority(RoleAuthorityDto dto)
    {
        //判断是否存在，如果存在就做更新
        var model = await _systemAuthorityRepository.GetAsync(x => x.RoleId == dto.RoleId);
        if (model is null)
        {
            model = _mapper.Map<SystemAuthority>(dto);
            await _systemAuthorityRepository.AddAsync(model);
        }
        else
        {
            _mapper.Map(dto, model);
            await _systemAuthorityRepository.UpdateAsync(model);
        }
    }

    /// <summary>
    /// 获取角色权限
    /// </summary>
    /// <param name="roleid"></param>
    /// <returns></returns>
    [Permission(EPermission.GetAuthority)]
    [HttpGet("getauthority")]
    public async Task<SystemAuthority?> GetAuthority(string roleid)
    {
        return await _systemAuthorityRepository.GetAsync(x => x.RoleId == roleid, HttpContext.RequestAborted);
    }

    #endregion

    #region 数据权限管理

    /// <summary>
    /// 新增数据权限
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [Permission(EPermission.AddDataAuthority)]
    [HttpPost("add-data-authority")]
    public async Task AddDataAuthority([FromBody] AddDataAuthorityDto dto)
    {
        var dataAuthority = _mapper.Map<SystemDataAuthority>(dto);
        await _systemDataAuthorityRepository.AddAsync(dataAuthority);
    }

    /// <summary>
    /// 修改数据权限
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [Permission(EPermission.UpdateDataAuthority)]
    [HttpPost("update-data-authority")]
    public async Task UpdateDataAuthority([FromBody] UpdateDataAuthorityDto dto)
    {
        var entity = await _systemDataAuthorityRepository.GetAsync(x => x.Id == dto.Id, HttpContext.RequestAborted);
        if (entity is null)
            throw UserFriendlyException.SameMessage("无效数据");

        _mapper.Map(dto, entity);
        await _systemDataAuthorityRepository.UpdateAsync(entity, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 删除数据权限
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    [Permission(EPermission.RemoveDataAuthority)]
    [HttpDelete("remove-data-authority/{id}")]
    public async Task RemoveDataAuthority(string id)
    {
        var entity = await _systemDataAuthorityRepository.GetAsync(id, HttpContext.RequestAborted);
        if (entity is null)
            throw UserFriendlyException.SameMessage("无效数据");

        await _systemDataAuthorityRepository.RemoveAsync(id);
    }

    /// <summary>
    /// 获取角色所有数据权限设置
    /// </summary>
    /// <param name="roleid"></param>
    /// <returns></returns>
    [Permission(EPermission.GetDataAuthorityByRole)]
    [HttpGet("getdataauthoritybyrole")]
    public async Task<IReadOnlyList<SystemDataAuthority>> GetDataAuthorityByRole(string roleid)
    {
        return await _systemDataAuthorityRepository.QueryAsync(x => x.RoleId == roleid);
    }

    #endregion

    #region private

    private void CheckHttpRequestSuccess(ApiResponse response, string msg)
    {
        if (response == null || !response.IsSuccess)
            throw new UserFriendlyException($"identity service request failed: {msg}");
    }

    #endregion
}