﻿using Hotline.FlowEngine.Definitions;
using Hotline.Repository.SqlSugar;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.Workflow;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Requests;
using MapsterMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;
using XF.Domain.Exceptions;

namespace Hotline.Api.Controllers;

/// <summary>
/// 工作流管理
/// </summary>
public class WorkflowController : BaseController
{
    private readonly IDefinitionDomainService _definitionDomainService;
    private readonly IDefinitionRepository _definitionRepository;
    private readonly IMapper _mapper;

    public WorkflowController(
        IDefinitionDomainService definitionDomainService,
        IDefinitionRepository definitionRepository,
        IMapper mapper)
    {
        _definitionDomainService = definitionDomainService;
        _definitionRepository = definitionRepository;
        _mapper = mapper;
    }

    /// <summary>
    /// 分页查询流程模板
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("definition")]
    public async Task<PagedDto<DefinitionDto>> QueryDefinitions([FromQuery] PagedRequest dto)
    {
        var result = await _definitionRepository.Queryable()
            .Where(d => d.Status == EDefinitionStatus.Temporary)
            .MergeTable()
            .Where(d => d.Status != EDefinitionStatus.Temporary)
            .GroupBy(d => d.Code)
            .OrderByDescending(d => d.Version)
            .Take(1)
            .ToPagedListAsync(dto.PageIndex, dto.PageSize, HttpContext.RequestAborted);

        var items = _mapper.Map<IReadOnlyList<DefinitionDto>>(result.Items);
        return new PagedDto<DefinitionDto>(result.Total, items);
    }

    /// <summary>
    /// 新增流程模板草稿
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPost("definition")]
    public async Task<string> AddDefinition([FromBody] AddDefinitionDto dto)
    {
        var definition = _mapper.Map<Definition>(dto);
        return await _definitionRepository.AddAsync(definition, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 更新流程模板草稿
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpPut("definition")]
    public async Task UpdateDefinition([FromBody] UpdateDefinitionDto dto)
    {
        var definition = await _definitionRepository.GetAsync(dto.Id, HttpContext.RequestAborted);
        if (definition == null)
            throw UserFriendlyException.SameMessage("无效模板编号");
        if (definition.Status == EDefinitionStatus.Temporary)
        {
            _mapper.Map(dto, definition);
            await _definitionRepository.UpdateAsync(definition, HttpContext.RequestAborted);
        }
        else
        {
            var newDefinition = _mapper.Map<Definition>(definition);
            _mapper.Map(dto, newDefinition);
            await _definitionRepository.AddAsync(newDefinition, HttpContext.RequestAborted);
        }
    }

    [HttpDelete("definition/{id}")]
    public async Task RemoveDefinition(string id)
    {
        var definition = await _definitionRepository.GetAsync(id, HttpContext.RequestAborted);
        if (definition == null) return;
        if (definition.Status != EDefinitionStatus.Temporary)
            throw new UserFriendlyException("已发布模板不能删除");
        await _definitionRepository.RemoveAsync(id, false, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 发布（将草稿发布为正式模板）
    /// </summary>
    /// <returns></returns>
    [HttpPost("definition/{id}/publish")]
    public async Task Publish(string id)
    {
        await _definitionDomainService.PublishAsync(id, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 发布（将草稿发布为正式模板）
    /// </summary>
    /// <returns></returns>
    [HttpPost("definition/publish")]
    public async Task Publish([FromBody] UpdateDefinitionDto dto)
    {
        var definition = _mapper.Map<Definition>(dto);
        await _definitionDomainService.PublishAsync(definition, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 启用流程模板
    /// </summary>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    [HttpPost("definition/enable")]
    public async Task Enable(string id)
    {
        var definition = await _definitionRepository.GetAsync(id, HttpContext.RequestAborted);
        if (definition == null)
            throw UserFriendlyException.SameMessage("无效模板编号");
        if (definition.Status == EDefinitionStatus.Temporary)
            throw UserFriendlyException.SameMessage("草稿需发布才能启用");
        if (definition.Status == EDefinitionStatus.Disable)
        {
            definition.Status = EDefinitionStatus.Enable;
            await _definitionRepository.UpdateAsync(definition, HttpContext.RequestAborted);
        }
    }

    /// <summary>
    /// 禁用流程模板
    /// </summary>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    [HttpPost("definition/disable")]
    public async Task Disable(string id)
    {
        var definition = await _definitionRepository.GetAsync(id, HttpContext.RequestAborted);
        if (definition == null)
            throw UserFriendlyException.SameMessage("无效模板编号");
        if (definition.Status == EDefinitionStatus.Temporary)
            throw UserFriendlyException.SameMessage("草稿无法禁用");
        if (definition.Status == EDefinitionStatus.Enable)
        {
            definition.Status = EDefinitionStatus.Disable;
            await _definitionRepository.UpdateAsync(definition, HttpContext.RequestAborted);
        }
    }

}