﻿using Hotline.Share.Enums;
using System.Linq.Expressions;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Entities;

namespace Hotline.Repository.SqlSugar.DataPermissions;

public class DataPermissionFilterBuilder : IDataPermissionFilterBuilder, IScopeDependency
{
    private readonly ISessionContext _sessionContext;

    public DataPermissionFilterBuilder(ISessionContext sessionContext, IDataPermissionManager dataPermissionManager)
    {
        DataPermissionManager = dataPermissionManager;
        _sessionContext = sessionContext;
    }

    public IDataPermissionManager DataPermissionManager { get; }

    public Expression<Func<TEntity, bool>> Build<TEntity>() where TEntity : class, IEntity<string>, IDataPermission, new()
    {
        var userId = _sessionContext.RequiredUserId;
        var scheme = DataPermissionManager.GetQueryFilter<TEntity>(_sessionContext);
        switch (scheme.QueryFilter)
        {
            case EAuthorityType.Create:
                return d => d.CreateUserId == userId;
            case EAuthorityType.Org:
                return d => d.AutoOrgCode == scheme.OrgCode;
            case EAuthorityType.OrgAndBelow:
                return d => d.AutoOrgCode.StartsWith(scheme.OrgCode);
            case EAuthorityType.All:
                return d => true;
            default:
                return d => false;
        }
    }

    public Expression<Func<TEntity, bool>> BuildIncludeFlowData<TEntity>() where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
    {
        var userId = _sessionContext.RequiredUserId;
        var scheme = DataPermissionManager.GetQueryFilter<TEntity>(_sessionContext);
        var (_,depCode, _, _) = DataPermissionManager.GetDataPermissionOptions();

        switch (scheme.QueryFilter)
        {
            case EAuthorityType.Create:
                return d => d.CreateUserId == userId || FlowDataFiltering(d, userId, depCode);
            case EAuthorityType.Org:
                return d => d.AutoOrgCode == scheme.OrgCode || FlowDataFiltering(d, userId, depCode);
            case EAuthorityType.OrgAndBelow:
                return d => d.AutoOrgCode.StartsWith(scheme.OrgCode) || FlowDataFiltering(d, userId, depCode);
            case EAuthorityType.All:
                return d => true;
            default:
                return d => FlowDataFiltering(d, userId, depCode);
        }
    }

    private static bool FlowDataFiltering<TEntity>(TEntity entity, string userId, string depCode) where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
    {
        if (entity.AssignUserIds.Contains(userId)) return true;
        foreach (var assignDepCode in entity.AssignDepCodes)
        {
            if (assignDepCode == depCode) return true;
            var baseDep = assignDepCode.Substring(0, 3);
            if (depCode.StartsWith(baseDep) && depCode.Length < assignDepCode.Length)
                return true;
        }

        return false;
    }
}