﻿using Hotline.Share.Enums.FlowEngine;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.FlowEngine.Definitions;

public class DefinitionDomainService : IDefinitionDomainService, IScopeDependency
{
    private readonly IDefinitionRepository _definitionRepository;

    public DefinitionDomainService(IDefinitionRepository definitionRepository)
    {
        _definitionRepository = definitionRepository;
    }

    /// <summary>
    /// 发布流程模板
    /// </summary>
    /// <param name="id"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public async Task PublishAsync(string id, CancellationToken cancellationToken)
    {
        var definitionTemp = await _definitionRepository.GetAsync(id, cancellationToken);
        ValidateDefinition(definitionTemp);

        var lastVersion = await GetLastVersionAsync(definitionTemp.Code, cancellationToken);
        definitionTemp.Version = lastVersion + 1;
        definitionTemp.Status = EDefinitionStatus.Enable;

        await _definitionRepository.UpdateAsync(definitionTemp, cancellationToken);
    }

    public async Task PublishAsync(Definition definition, CancellationToken cancellationToken)
    {
        var temp = await _definitionRepository.GetAsync(d => d.Code == definition.Code && d.Status == EDefinitionStatus.Temporary, cancellationToken);
        if (temp is null)
            await _definitionRepository.AddAsync(definition, cancellationToken);
        await PublishAsync(temp.Id, cancellationToken);
    }

    public async Task<int> GetLastVersionAsync(string code, CancellationToken cancellationToken)
    {
        var definition = await GetLastVersionDefinitionAsync(code, cancellationToken);
        return definition?.Version ?? 0;
    }

    public async Task<Definition?> GetLastVersionDefinitionAsync(string code, CancellationToken cancellationToken)
    {
        return await _definitionRepository.Queryable()
            .Where(d => d.Code == code)
            .OrderByDescending(d => d.Version)
            .Take(1)
            .FirstAsync();

        //var definitions = await _definitionRepository.GetAllAsync(d => d.Code == code, cancellationToken);
        //if (!definitions.Any()) return null;
        //return definitions.MaxBy(d => d.Version);
    }

    /// <summary>
    /// 未开启的流程，查找第2个节点的模板配置信息，用作开始流程的时候传参
    /// </summary>
    /// <param name="definitionCode"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public async Task<IReadOnlyList<NextStepDefine>> GetInitNextStepsAsync(string definitionCode, CancellationToken cancellationToken)
    {
        var definition = await GetLastVersionDefinitionAsync(definitionCode, cancellationToken);
        return definition.Steps.First(d => d.StepType == EStepType.Start).NextSteps;
    }

    #region private
    private void ValidateDefinition(Definition definition)
    {
        if (definition is null)
            throw new UserFriendlyException("无效 Definition Id");
        if (definition.Code == null)
            throw UserFriendlyException.SameMessage("无效Code");
        if (definition.Status is not EDefinitionStatus.Temporary)
            throw UserFriendlyException.SameMessage("只能发布草稿状态的模板");
        if (!definition.Steps.Any())
            throw UserFriendlyException.SameMessage("未配置流程节点");
        if (definition.Steps.Any(d => d.StepType == EStepType.Start))
            throw UserFriendlyException.SameMessage("未配置开始节点");
        if (definition.Steps.Any(d => d.StepType == EStepType.End))
            throw UserFriendlyException.SameMessage("未配置结束节点");
        if (definition.Steps.Count(d => d.StepType == EStepType.Start) > 1)
            throw UserFriendlyException.SameMessage("开始节点只能有一个");
        if (definition.Steps.Count(d => d.StepType == EStepType.End) > 1)
            throw UserFriendlyException.SameMessage("结束节点只能有一个");

        foreach (var step in definition.Steps)
        {
            if (step.StepType != EStepType.End && !step.NextSteps.Any())
                throw UserFriendlyException.SameMessage("其中有节点未配置下一节点");
        }

        var startStep = definition.Steps.First(d => d.StepType == EStepType.Start);
        if (startStep.NextSteps.Count != 1)
            throw UserFriendlyException.SameMessage("开始节点只能配置一个下一节点");
    }

    #endregion
}