﻿using Hotline.FlowEngine.Definitions;
using Hotline.FlowEngine.Notifies;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.FlowEngine;
using MapsterMapper;
using MediatR;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.FlowEngine.Workflows
{
    public class WorkflowDomainService : IWorkflowDomainService, IScopeDependency
    {
        private readonly IWorkflowRepository _workflowRepository;
        private readonly IMapper _mapper;
        private readonly IMediator _mediator;

        public WorkflowDomainService(
            IWorkflowRepository workflowRepository,
            IMapper mapper,
            IMediator mediator)
        {
            _workflowRepository = workflowRepository;
            _mapper = mapper;
            _mediator = mediator;
        }

        /// <summary>
        /// 返回前一级
        /// </summary>
        /// <returns></returns>
        public async Task PreviousAsync(PreviousDto dto, CancellationToken cancellationToken)
        {
            //var workflow = await _workflowRepository.GetAsync(dto.WorkflowId, cancellationToken);
            var workflow = await GetWorkflowAsync(dto.WorkflowId, cancellationToken);

            var (stepBox, currentStep) = GetStep(workflow.StepBoxes, dto.DepCode, dto.UserId);
            if (currentStep.StepType is EStepType.Start)
                throw new UserFriendlyException("当前流程已流转到开始步骤");

            //traces新增记录
            var trace = CreateTrace(workflow.Id, currentStep, dto, true);
            workflow.Traces.Add(trace);

            //取消上一节点stepBox.endtime
            var (previousStepBox, previousStep) = GetStep(workflow.StepBoxes, currentStep.PreviousId);
            previousStep.EndTime = null;
            previousStepBox.EndTime = null;

            //删除steps节点
            stepBox.Steps.Remove(currentStep);

            await _workflowRepository.UpdateNavigateAsync(workflow, cancellationToken);

            await _mediator.Publish(new PreviousStepNotify(dto), cancellationToken);
        }

        /// <summary>
        /// 前往下一节点
        /// </summary>
        /// <returns></returns>
        public async Task NextAsync(ApproverDto dto, CancellationToken cancellationToken)
        {
            var workflow = await _workflowRepository.GetAsync(dto.WorkflowId, cancellationToken);
            if (workflow is null)
                throw new UserFriendlyException("无效workflowId");

            var (stepBox, currentStep) = GetStep(workflow.StepBoxes, dto.DepCode, dto.UserId);
            if (currentStep.StepType is EStepType.End)
                throw new UserFriendlyException("当前流程已流转到最终步骤");

            currentStep.EndTime = DateTime.Now;
            _mapper.Map(dto, currentStep);

            var selectedStep = currentStep.NextSteps.First(d => d.Code == dto.NextStepCode);
            selectedStep.Selected = true;
            if (stepBox.Steps.All(d => d.EndTime.HasValue))
                stepBox.EndTime = currentStep.EndTime;

            //创建下一节点
            //依据nextStepCode，检查box是否存在，不存在则在模板中找到对应define创建box，存在则在box中新增step
            var stepDefine = workflow.Definition.Steps.FirstOrDefault(d => d.Code == dto.NextStepCode);
            if (stepDefine is null)
                throw UserFriendlyException.SameMessage($"模板配置中未找到对应节点, code: {dto.NextStepCode}");
            var existsStepBox = workflow.StepBoxes.FirstOrDefault(d => d.Code == dto.NextStepCode);
            if (existsStepBox == null)
            {
                var steps = CreateStepByStepDefine(stepDefine, dto.Handlers, currentStep.Id);
                existsStepBox = CreateStepBox(stepDefine, steps);
                workflow.StepBoxes.Add(existsStepBox);
            }
            else
            {
                var steps = CreateStepByStepDefine(stepDefine, dto.Handlers, currentStep.Id);
                existsStepBox.Steps.AddRange(steps);
            }

            //trace
            var trace = CreateTrace(workflow.Id, currentStep, dto);
            workflow.Traces.Add(trace);

            await _workflowRepository.UpdateNavigateAsync(workflow, cancellationToken);
            
            await _mediator.Publish(new NextStepNotify(dto), cancellationToken);
        }

        /// <summary>
        /// 根据操作人获取当前节点
        /// </summary>
        /// <returns></returns>
        public async Task<(WorkflowStep, WorkflowStep)> GetStepAsync(string workflowId, string depCode, string userId, CancellationToken cancellationToken)
        {
            var workflow = await _workflowRepository.GetAsync(workflowId, cancellationToken);
            if (workflow is null)
                throw new UserFriendlyException("无效workflowId");

            return GetStep(workflow.StepBoxes, depCode, userId);
        }

        public (WorkflowStep, WorkflowStep) GetStep(List<WorkflowStep> stepBoxes, string depCode, string userId)
        {
            foreach (var stepBox in stepBoxes)
            {
                foreach (var step in stepBox.Steps)
                {
                    if (step.HandlerId == depCode || step.HandlerId == userId)
                        return (stepBox, step);
                }
            }

            throw new UserFriendlyException("未找到对应节点");
        }

        /// <summary>
        /// 获取可能前往的下一节点
        /// </summary>
        /// <returns></returns>
        public async Task<IReadOnlyList<NextStepDefine>> GetNextStepsAsync(QueryWorkflowDto dto, CancellationToken cancellationToken)
        {
            var (_, currentStep) = await GetStepAsync(dto.WorkflowId, dto.DepCode, dto.UserId, cancellationToken);
            //获取人工判定的所有下一节点
            return currentStep.NextSteps.Where(d => string.IsNullOrEmpty(d.Predicate)).ToList();
        }

        public Workflow CreateWorkflow(Definition definition, StartWorkflowDto dto)
        {
            var workflow = new Workflow
            {

                Definition = definition,
                Status = EWorkflowStatus.Runnable,
            };

            var startStep = definition.Steps.First(d => d.StepType == EStepType.Start);
            if (startStep.NextSteps.Count > 1)
                throw new UserFriendlyException("开始节点存在多个下一节点");

            //start节点后的首个节点
            var firstStep = startStep.NextSteps.First();
            if (firstStep.HandlerType == EHandlerType.AssignUser || firstStep.HandlerType == EHandlerType.AssignDep)
            {
                var steps = CreateStepByStepDefine(firstStep, firstStep.HandlerClassifies, "StartStep");
                workflow.StepBoxes.Add(CreateStepBox(firstStep, steps));
            }
            else
            {
                if (dto.Handlers.Any())
                {
                    var steps = CreateStepByStepDefine(firstStep, dto.Handlers, "StartStep");
                    workflow.StepBoxes.Add(CreateStepBox(firstStep, steps));
                }
                else
                {
                    throw new UserFriendlyException("未指定节点处理者");
                }
            }

            return workflow;
        }

        public void AddStartTrace(Workflow workflow, StartWorkflowDto dto)
        {
            var startStep = workflow.Definition.Steps.First(d => d.StepType == EStepType.Start);
            var trace = _mapper.Map<WorkflowTrace>(startStep);
            _mapper.Map(dto, trace);
            trace.WorkflowId = workflow.Id;
            workflow.Traces.Add(trace);
        }

        #region private

        private (WorkflowStep, WorkflowStep) GetStep(List<WorkflowStep> stepBoxes, string stepId)
        {
            foreach (var stepBox in stepBoxes)
            {
                foreach (var step in stepBox.Steps)
                {
                    if (step.Id == stepId)
                        return (stepBox, step);
                }
            }

            throw new UserFriendlyException("未找到对应节点");
        }

        private List<WorkflowStep> CreateStepByStepDefine(StepBasic stepBasic, List<string> handlers, string previousId)
        {
            return handlers.Select(d =>
            {
                var step = _mapper.Map<WorkflowStep>(stepBasic);
                step.Id = Guid.NewGuid().ToString();
                step.HandlerId = d;
                if (!string.IsNullOrEmpty(previousId))
                    step.PreviousId = previousId;
                return step;
            }).ToList();
        }

        private WorkflowStep CreateStepBox(StepBasic stepBasic, List<WorkflowStep>? steps = null)
        {
            var stepBox = _mapper.Map<WorkflowStep>(stepBasic);
            if (steps != null && steps.Any())
                stepBox.Steps = steps;
            return stepBox;
        }

        private WorkflowTrace CreateTrace(string workflowId, WorkflowStep currentStep, ApproverDto dto, bool isBackward = false)
        {
            var trace = _mapper.Map<WorkflowTrace>(currentStep);
            _mapper.Map(dto, trace);
            trace.WorkflowId = workflowId;
            trace.IsBackward = isBackward;
            return trace;
        }

        private WorkflowTrace CreateTrace(string workflowId, WorkflowStep currentStep, PreviousDto dto, bool isBackward = false)
        {
            var trace = _mapper.Map<WorkflowTrace>(currentStep);
            _mapper.Map(dto, trace);
            trace.WorkflowId = workflowId;
            trace.IsBackward = isBackward;
            return trace;
        }

        private async Task<Workflow> GetWorkflowAsync(string workflowId, CancellationToken cancellationToken)
        {
            var workflow = await _workflowRepository.Queryable()
                .Includes(d => d.Definition)
                .Includes(d => d.StepBoxes)
                .Includes(d => d.Traces)
                .FirstAsync(d => d.Id == workflowId);
            if (workflow is null)
                throw new UserFriendlyException("无效workflowId");
            return workflow;
        }

        #endregion

    }
}
