﻿using SqlSugar;
using XF.Domain.Entities;
using XF.Domain.Events;

namespace XF.Domain.Repository;

public abstract class Entity : IEntity<string>, IDomainEvents, IDataPermission
{
    private List<IAppNotification> _domainEvents = new();

    public string Id { get; set; }

    /// <summary>
    /// 组织Id
    /// </summary>
    [SugarColumn(ColumnDescription ="组织Id",IsNullable = true)]
    public string OrgId { get; private set; }
    /// <summary>
    /// 组织编码
    /// </summary>
    [SugarColumn(ColumnDescription = "组织编码", IsNullable = true)]
    public string AutoOrgCode { get; private set; }
    /// <summary>
    /// 创建人
    /// </summary>
    [SugarColumn(ColumnDescription = "创建人", IsNullable = true)]
    public string CreateUserId { get; private set; }

    /// <summary>
    /// 赋值部门Id
    /// </summary>
    [SugarColumn(ColumnDescription = "数据权限区域Id", IsNullable = true)]
    public string AreaId { get; private set; }

    public void CreateDataPermission(string orgId,string departmentCode, string creatorId, string? areaId)
    {
        OrgId = OrgId;
        AutoOrgCode = departmentCode;
        CreateUserId = creatorId;
        AreaId = areaId ?? string.Empty;
    }

    public void AddEvent(IAppNotification notification)
    {
        if (_domainEvents.Contains(notification)) return;
        _domainEvents.Add(notification);
    }

    public void ClearEvents() => _domainEvents.Clear();

    public ICollection<IAppNotification> QueryAllEvents() => _domainEvents;
}

/// <summary>
/// 实体（带有创建时间）
/// </summary>
public abstract class CreationEntity : Entity, IHasCreationTime
{
    /// <summary>
    /// 创建时间
    /// </summary>
    [SugarColumn(ColumnDescription = "创建时间")]
    public DateTime CreationTime { get; set; }
}

/// <summary>
/// 实体（带有软删除）
/// </summary>
public abstract class SoftDeleteEntity : Entity, IHasDeletionTime, ISoftDelete
{
    /// <summary>
    /// 删除时间
    /// </summary>
    [SugarColumn(ColumnDescription = "删除时间")]
    public DateTime? DeletionTime { get; private set; }

    [SugarColumn(ColumnDescription = "是否删除")]
    public bool IsDeleted { get; private set; }

    public void SoftDelete()
    {
        IsDeleted = true;
        DeletionTime = DateTime.Now;
    }
}

/// <summary>
/// 实体（带有创建时间和软删除）
/// </summary>
public abstract class CreationSoftDeleteEntity : CreationEntity, IHasDeletionTime, ISoftDelete
{
    [SugarColumn(ColumnDescription = "是否删除")]
    public bool IsDeleted { get; private set; }

    /// <summary>
    /// 删除时间
    /// </summary>
    [SugarColumn(ColumnDescription = "删除时间")]
    public DateTime? DeletionTime { get; set; }

    public void SoftDelete()
    {
        IsDeleted = true;
        DeletionTime = DateTime.Now;
    }

    public void Recover() => IsDeleted = false;
}

public abstract class CreationModificationEntity : CreationEntity, IHasModificationTime
{
    /// <summary>
    /// 最近更新时间
    /// </summary>
    [SugarColumn(ColumnDescription = "最近更新时间")]
    public DateTime? LastModificationTime { get; set; }

    public void Modified() => LastModificationTime = DateTime.Now;
}

/// <summary>
/// 全状态实体(创建时间、更新时间、软删除)
/// </summary>
public abstract class FullStateEntity : CreationSoftDeleteEntity, IHasModificationTime, IWorkflow
{
    /// <summary>
    /// 最近更新时间
    /// </summary>
    [SugarColumn(ColumnDescription = "最近更新时间")]
    public DateTime? LastModificationTime { get; set; }

    public void Modified() => LastModificationTime = DateTime.Now;

    [SugarColumn(IsNullable = true)]
    public string? WorkflowId { get; set; }

    [SugarColumn(ColumnDataType = "varchar(4000)", IsJson = true)]
    public List<string> AssignDepCodes { get; set; } = new();

    [SugarColumn(ColumnDataType = "varchar(4000)", IsJson = true)]
    public List<string> AssignUserIds { get; set; } = new();
}

